@@ -1,4 +1,4 @@
-// Copyright 2009-2020 Intel Corporation
+// Copyright 2009-2021 Intel Corporation
 // SPDX-License-Identifier: Apache-2.0
 
 #include <cassert>
@@ -11,34 +11,36 @@
 #define CATCH_CONFIG_FAST_COMPILE
 #include "catch.hpp"
 
+OIDN_NAMESPACE_USING
 using namespace oidn;
 
 void setFilterImage(FilterRef& filter, const char* name, ImageBuffer& image)
 {
   Format format = Format::Undefined;
-  switch (image.getChannels())
+  switch (image.numChannels)
   {
   case 1: format = Format::Float;  break;
   case 2: format = Format::Float2; break;
   case 3: format = Format::Float3; break;
   case 4: format = Format::Float4; break;
-  default: assert(0);
+  default:
+    assert(0);
   }
 
-  filter.setImage(name, image.getData(), format, image.getWidth(), image.getHeight());
+  filter.setImage(name, image.data(), format, image.width, image.height);
 }
 
 ImageBuffer makeConstImage(int W, int H, int C = 3, float value = 0.5f)
 {
   ImageBuffer image(W, H, C);
-  for (int i = 0; i < image.getSize(); ++i)
+  for (size_t i = 0; i < image.size(); ++i)
     image[i] = value;
   return image;
 }
 
 bool isBetween(const ImageBuffer& image, float a, float b)
 {
-  for (int i = 0; i < image.getSize(); ++i)
+  for (size_t i = 0; i < image.size(); ++i)
     if (!std::isfinite(image[i]) || image[i] < a || image[i] > b)
       return false;
   return true;
@@ -76,7 +78,7 @@ void sanitizationTest(DeviceRef& device, bool hdr, float value)
 
 TEST_CASE("image sanitization", "[sanitization]")
 {
-  DeviceRef device = oidn::newDevice();
+  DeviceRef device = newDevice();
   REQUIRE(device);
   device.commit();
   REQUIRE(device.getError() == Error::None);
@@ -156,7 +158,7 @@ void progressTest(DeviceRef& device, double nMax = 1000)
 
 TEST_CASE("progress monitor callback", "[progress]")
 {
-  DeviceRef device = oidn::newDevice();
+  DeviceRef device = newDevice();
   REQUIRE(device);
   device.commit();
   REQUIRE(device.getError() == Error::None);
