@@ -18,6 +18,7 @@
 
 #include "dnnl_thread.hpp"
 #include "dnnl_traits.hpp"
+#include "stream.hpp"
 #include "type_helpers.hpp"
 #include "utils.hpp"
 
@@ -185,15 +186,16 @@ void typed_zero_pad_generic_blocked(
 }
 
 template <data_type_t dt>
-status_t memory_t::typed_zero_pad() const {
-    const memory_desc_wrapper mdw(md());
+status_t typed_zero_pad(const memory_t *memory, stream_t *stream) {
+    memory_storage_t *memory_storage = memory->memory_storage();
+    const memory_desc_wrapper mdw(memory->md());
 
     if (mdw.format_kind() != format_kind::blocked) return unimplemented;
 
     if (mdw.nelems(false) == mdw.nelems(true)) return success;
 
     void *mapped_ptr = nullptr;
-    status_t status = memory_storage()->map_data(&mapped_ptr);
+    status_t status = memory_storage->map_data(&mapped_ptr, stream);
     assert(status == status::success);
 
     auto *data = (typename prec_traits<dt>::type *)mapped_ptr;
@@ -212,7 +214,7 @@ status_t memory_t::typed_zero_pad() const {
     do { \
         if (blksize == blksize_) { \
             typed_zero_pad_blk<dt, blk_kind, blksize_>(mdw, data); \
-            status = memory_storage()->unmap_data(mapped_ptr); \
+            status = memory_storage->unmap_data(mapped_ptr, stream); \
             assert(status == status::success); \
             return success; \
         } \
@@ -263,27 +265,48 @@ status_t memory_t::typed_zero_pad() const {
     // the last line of defence
     typed_zero_pad_generic_blocked<dt>(mdw, data);
 
-    status = memory_storage()->unmap_data(mapped_ptr);
+    status = memory_storage->unmap_data(mapped_ptr, stream);
     assert(status == status::success);
 
     MAYBE_UNUSED(status);
     return success;
 }
 
-status_t memory_t::zero_pad() const {
+static status_t zero_pad(const memory_t *memory, stream_t *stream) {
+    memory_desc_wrapper mdw(memory->md());
+    switch (mdw.data_type()) {
+        case f16: return typed_zero_pad<f16>(memory, stream);
+        case bf16: return typed_zero_pad<bf16>(memory, stream);
+        case f32: return typed_zero_pad<f32>(memory, stream);
+        case s32: return typed_zero_pad<s32>(memory, stream);
+        case s8: return typed_zero_pad<s8>(memory, stream);
+        case u8: return typed_zero_pad<u8>(memory, stream);
+        default: assert(!"memory is undefined"); return unimplemented;
+    }
+    return unimplemented;
+}
+
+status_t stream_t::zero_pad(const memory_t *memory) {
+    return ::zero_pad(memory, this);
+}
+
+status_t memory_t::zero_pad(stream_t *stream) const {
     memory_desc_wrapper mdw(md());
     const bool skip_zeroing = false || memory_storage()->is_null()
             || mdw.is_zero() || !mdw.is_blocking_desc();
     if (skip_zeroing) return success;
 
-    switch (mdw.data_type()) {
-        case f16: return typed_zero_pad<f16>();
-        case bf16: return typed_zero_pad<bf16>();
-        case f32: return typed_zero_pad<f32>();
-        case s32: return typed_zero_pad<s32>();
-        case s8: return typed_zero_pad<s8>();
-        case u8: return typed_zero_pad<u8>();
-        default: assert(!"memory is undefined"); return unimplemented;
+    status_t status;
+    if (stream == nullptr) {
+        engine_t *engine;
+        engine = memory_storage()->engine();
+        stream = engine->service_stream();
     }
-    return unimplemented;
+
+    if (stream != nullptr)
+        status = stream->zero_pad(this);
+    else
+        status = ::zero_pad(this, stream);
+
+    return status;
 }
