@@ -18,15 +18,19 @@
 #include <map>
 #include <vector>
 
-#include "cpu_engine.hpp"
-#include "cpu_reorder_pd.hpp"
-#include "memory.hpp"
-#include "type_helpers.hpp"
+#include "common/memory.hpp"
+#include "common/type_helpers.hpp"
 
-#include "cpu/jit_uni_reorder.hpp"
-#include "cpu/rnn/rnn_reorders.hpp"
+#include "cpu/cpu_engine.hpp"
+#include "cpu/cpu_reorder_pd.hpp"
+
+//#include "cpu/rnn/rnn_reorders.hpp"
 #include "cpu/simple_reorder.hpp"
-#include "cpu/wino_reorder.hpp"
+
+#if DNNL_X64
+#include "cpu/x64/jit_uni_reorder.hpp"
+//#include "cpu/x64/wino_reorder.hpp"
+#endif
 
 namespace dnnl {
 namespace impl {
@@ -96,7 +100,7 @@ static const impl_list_map_t regular_impl_list_map {
     {{f32, bf16, 0}, {
         rnn_weights_reorder_t<f32, bf16>::pd_t::create,
 
-        jit_uni_reorder_create,
+        DNNL_X64_ONLY(x64::jit_uni_reorder_create,)
 
         REG_SR_BIDIR(f32, any, bf16, nChw16c),
         REG_SR_BIDIR(f32, any, bf16, nCdhw16c),
@@ -127,7 +131,7 @@ static const impl_list_map_t regular_impl_list_map {
     {{f32, f32, 0}, {
         REG_FAST_DIRECT_COPY_F32_F32_COMMA
 
-        jit_uni_reorder_create,
+        DNNL_X64_ONLY(x64::jit_uni_reorder_create,)
 
         REG_SR(f32, any, f32, any, fmt_order::any, spec::reference),
 
@@ -135,37 +139,37 @@ static const impl_list_map_t regular_impl_list_map {
     }},
     /*
     {{f32, f32, 3}, {
-       REG_FAST_DIRECT_COPY_F32_F32_COMMA
+        REG_FAST_DIRECT_COPY_F32_F32_COMMA
 
-       jit_uni_reorder_create,
+        DNNL_X64_ONLY(x64::jit_uni_reorder_create,)
 
-       REG_SR_BIDIR(f32, any, f32, nCw16c),
-       REG_SR_BIDIR(f32, any, f32, nCw8c),
-       REG_SR_BIDIR(f32, any, f32, nCw4c),
+        REG_SR_BIDIR(f32, any, f32, nCw16c),
+        REG_SR_BIDIR(f32, any, f32, nCw8c),
+        REG_SR_BIDIR(f32, any, f32, nCw4c),
 
-       REG_SR_BIDIR(f32, nCw4c, f32, nCw16c),
-       REG_SR_BIDIR(f32, nCw8c, f32, nCw16c),
+        REG_SR_BIDIR(f32, nCw4c, f32, nCw16c),
+        REG_SR_BIDIR(f32, nCw8c, f32, nCw16c),
 
-       REG_SR_BIDIR(f32, any, f32, OIw4i4o),
-       REG_SR_BIDIR(f32, any, f32, OIw4o4i),
-       REG_SR_BIDIR(f32, any, f32, OIw8i8o),
-       REG_SR_BIDIR(f32, any, f32, OIw8o8i),
+        REG_SR_BIDIR(f32, any, f32, OIw4i4o),
+        REG_SR_BIDIR(f32, any, f32, OIw4o4i),
+        REG_SR_BIDIR(f32, any, f32, OIw8i8o),
+        REG_SR_BIDIR(f32, any, f32, OIw8o8i),
 
-       REG_SR_BIDIR(f32, any, f32, OIw16o16i),
-       REG_SR_BIDIR(f32, any, f32, OIw16i16o),
-       REG_SR_BIDIR(f32, any, f32, IOw16o16i),
+        REG_SR_BIDIR(f32, any, f32, OIw16o16i),
+        REG_SR_BIDIR(f32, any, f32, OIw16i16o),
+        REG_SR_BIDIR(f32, any, f32, IOw16o16i),
 
-       REG_SR(f32, any, f32, any, fmt_order::any, spec::reference),
+        REG_SR(f32, any, f32, any, fmt_order::any, spec::reference),
 
-       nullptr,
+        nullptr,
     }},
     */
     {{f32, f32, 4}, {
-        wino_reorder_t<f32, f32>::pd_t::create,
+        //DNNL_X64_ONLY(x64::wino_reorder_t<f32, f32>::pd_t::create,)
 
         REG_FAST_DIRECT_COPY_F32_F32_COMMA
 
-        jit_uni_reorder_create,
+        DNNL_X64_ONLY(x64::jit_uni_reorder_create,)
 
         REG_SR_BIDIR(f32, any, f32, nChw16c),
         REG_SR_BIDIR(f32, any, f32, nChw8c),
@@ -206,12 +210,12 @@ static const impl_list_map_t regular_impl_list_map {
     }},
     /*
     {{f32, f32, 5}, {
-        wino_reorder_t<f32, f32>::pd_t::create,
+        DNNL_X64_ONLY(x64::wino_reorder_t<f32, f32>::pd_t::create,)
         rnn_weights_reorder_t<f32, f32>::pd_t::create,
 
         REG_FAST_DIRECT_COPY_F32_F32_COMMA
 
-        jit_uni_reorder_create,
+        DNNL_X64_ONLY(x64::jit_uni_reorder_create,)
 
         REG_SR_BIDIR(f32, any, f32, nCdhw16c),
         REG_SR_BIDIR(f32, any, f32, nCdhw8c),
@@ -256,7 +260,7 @@ static const impl_list_map_t regular_impl_list_map {
     {{f32, f32, 6}, {
         REG_FAST_DIRECT_COPY_F32_F32_COMMA
 
-        jit_uni_reorder_create,
+        DNNL_X64_ONLY(x64::jit_uni_reorder_create,)
 
         REG_SR_BIDIR(f32, any, f32, gOIdhw4i4o),
         REG_SR_BIDIR(f32, any, f32, gOIdhw4o4i),
@@ -280,7 +284,7 @@ static const impl_list_map_t regular_impl_list_map {
     {{f32, s32, 0}, {
         REG_FAST_DIRECT_COPY_COMMA(f32, s32)
 
-        jit_uni_reorder_create,
+        DNNL_X64_ONLY(x64::jit_uni_reorder_create,)
 
         REG_SR_BIDIR(f32, any, s32, nChw16c),
 
@@ -291,12 +295,12 @@ static const impl_list_map_t regular_impl_list_map {
 
     // f32 -> s8
     {{f32, s8, 0}, {
-        wino_reorder_t<f32, s8>::pd_t::create,
+        DNNL_X64_ONLY(x64::wino_reorder_t<f32, s8>::pd_t::create,)
         rnn_weights_reorder_s8_t<f32>::pd_t::create,
 
         REG_FAST_DIRECT_COPY_COMMA(f32, s8)
 
-        jit_uni_reorder_create,
+        DNNL_X64_ONLY(x64::jit_uni_reorder_create,)
 
         REG_SR_BIDIR(f32, any, s8, nChw16c),
         REG_SR_BIDIR(f32, any, s8, OIhw4i16o4i),
@@ -313,7 +317,7 @@ static const impl_list_map_t regular_impl_list_map {
 
         REG_FAST_DIRECT_COPY_COMMA(f32, u8)
 
-        jit_uni_reorder_create,
+        DNNL_X64_ONLY(x64::jit_uni_reorder_create,)
 
         REG_SR_BIDIR(f32, any, u8, nChw16c),
 
@@ -326,7 +330,7 @@ static const impl_list_map_t regular_impl_list_map {
     {{bf16, data_type::undef, 0}, {
         rnn_weights_reorder_t<bf16, bf16>::pd_t::create,
 
-        jit_uni_reorder_create,
+        DNNL_X64_ONLY(x64::jit_uni_reorder_create,)
 
         REG_SR_BIDIR(bf16, any, f32, nChw16c),
         REG_SR_BIDIR(bf16, any, f32, nCdhw16c),
@@ -358,7 +362,7 @@ static const impl_list_map_t regular_impl_list_map {
         REG_FAST_DIRECT_COPY_COMMA(s32, s8)
         REG_FAST_DIRECT_COPY_COMMA(s32, u8)
 
-        jit_uni_reorder_create,
+        DNNL_X64_ONLY(x64::jit_uni_reorder_create,)
 
         REG_SR_BIDIR(s32, any, f32, nChw16c),
         REG_SR_BIDIR(s32, any, s32, nChw16c),
@@ -375,31 +379,31 @@ static const impl_list_map_t regular_impl_list_map {
 
     // s8 ->
     {{s8, data_type::undef, 0}, {
-       rnn_weights_reorder_s8_t<s8>::pd_t::create,
+        rnn_weights_reorder_s8_t<s8>::pd_t::create,
 
-       REG_FAST_DIRECT_COPY_COMMA(s8, f32)
-       REG_FAST_DIRECT_COPY_COMMA(s8, s32)
-       REG_FAST_DIRECT_COPY_COMMA(s8, s8)
-       REG_FAST_DIRECT_COPY_COMMA(s8, u8)
+        REG_FAST_DIRECT_COPY_COMMA(s8, f32)
+        REG_FAST_DIRECT_COPY_COMMA(s8, s32)
+        REG_FAST_DIRECT_COPY_COMMA(s8, s8)
+        REG_FAST_DIRECT_COPY_COMMA(s8, u8)
 
-       jit_uni_reorder_create,
+        DNNL_X64_ONLY(x64::jit_uni_reorder_create,)
 
-       REG_SR_BIDIR(s8, any, f32, nChw16c),
-       REG_SR_BIDIR(s8, any, s32, nChw16c),
-       REG_SR_BIDIR(s8, any, s8, nChw16c),
-       REG_SR_BIDIR(s8, any, u8, nChw16c),
+        REG_SR_BIDIR(s8, any, f32, nChw16c),
+        REG_SR_BIDIR(s8, any, s32, nChw16c),
+        REG_SR_BIDIR(s8, any, s8, nChw16c),
+        REG_SR_BIDIR(s8, any, u8, nChw16c),
 
-       REG_SR_BIDIR(s8, any, f32, OIhw4i16o4i),
-       REG_SR_BIDIR(s8, any, s8, OIhw4i16o4i),
-       REG_SR_BIDIR(s8, any, f32, gOIhw4i16o4i),
-       REG_SR_BIDIR(s8, any, s8, gOIhw4i16o4i),
+        REG_SR_BIDIR(s8, any, f32, OIhw4i16o4i),
+        REG_SR_BIDIR(s8, any, s8, OIhw4i16o4i),
+        REG_SR_BIDIR(s8, any, f32, gOIhw4i16o4i),
+        REG_SR_BIDIR(s8, any, s8, gOIhw4i16o4i),
 
-       REG_SR(s8, any, f32, any, fmt_order::any, spec::reference),
-       REG_SR(s8, any, s32, any, fmt_order::any, spec::reference),
-       REG_SR(s8, any, s8, any, fmt_order::any, spec::reference),
-       REG_SR(s8, any, u8, any, fmt_order::any, spec::reference),
+        REG_SR(s8, any, f32, any, fmt_order::any, spec::reference),
+        REG_SR(s8, any, s32, any, fmt_order::any, spec::reference),
+        REG_SR(s8, any, s8, any, fmt_order::any, spec::reference),
+        REG_SR(s8, any, u8, any, fmt_order::any, spec::reference),
 
-       nullptr,
+        nullptr,
     }},
 
     // u8 ->
@@ -409,7 +413,7 @@ static const impl_list_map_t regular_impl_list_map {
         REG_FAST_DIRECT_COPY_COMMA(u8, s8)
         REG_FAST_DIRECT_COPY_COMMA(u8, u8)
 
-        jit_uni_reorder_create,
+        DNNL_X64_ONLY(x64::jit_uni_reorder_create,)
 
         REG_SR_BIDIR(u8, any, f32, nChw16c),
         REG_SR_BIDIR(u8, any, s32, nChw16c),
