@@ -30,9 +30,14 @@ namespace concat {
 void check_correctness(const settings_t &s) {
     for_(const auto &i_sdt : s.sdt)
     for_(const auto &i_ddt : s.ddt)
-    for_(const auto &i_stag : s.stag)
+    for_(const auto &i_stag_ : s.stag)
     for_(const auto &i_dtag : s.dtag)
     for (const auto &i_axis : s.axis) {
+
+        // broadcast tag if needed
+        auto i_stag = i_stag_;
+        if (i_stag.size() == 1) i_stag.assign(s.sdims.size(), i_stag[0]);
+
         if (s.sdims.size() != i_stag.size()) // want 1:1 match of sdims and tag
             SAFE_V(FAIL);
 
@@ -47,7 +52,7 @@ void check_correctness(const settings_t &s) {
         int status = doit(&p, &res);
 
         bool want_perf_report = false;
-        parse_result(res, want_perf_report, s.allow_unimpl, status, pstr);
+        parse_result(res, want_perf_report, status, pstr);
 
         if (want_perf_report && bench_mode & PERF) {
             perf_report_t pr(s.perf_template);
@@ -62,15 +67,15 @@ int bench(int argc, char **argv) {
     driver_name = "concat";
     using namespace parser;
     static settings_t s;
+    static const settings_t def {};
     for (; argc > 0; --argc, ++argv) {
         const bool parsed_options = parse_bench_settings(argv[0])
                 || parse_batch(bench, argv[0])
-                || parse_dt(s.sdt, argv[0], "sdt")
-                || parse_dt(s.ddt, argv[0], "ddt")
-                || parse_multi_tag(s.stag, argv[0])
-                || parse_tag(s.dtag, argv[0], "dtag")
-                || parse_axis(s.axis, argv[0])
-                || parse_allow_unimpl(s.allow_unimpl, argv[0])
+                || parse_dt(s.sdt, def.sdt, argv[0], "sdt")
+                || parse_dt(s.ddt, def.ddt, argv[0], "ddt")
+                || parse_multi_tag(s.stag, def.stag, argv[0])
+                || parse_tag(s.dtag, def.dtag, argv[0], "dtag")
+                || parse_axis(s.axis, def.axis, argv[0])
                 || parse_perf_template(s.perf_template, s.perf_template_def,
                         s.perf_template_csv, argv[0])
                 || parse_reset(s, argv[0]);
@@ -79,10 +84,6 @@ int bench(int argc, char **argv) {
 
             parse_multi_dims(s.sdims, argv[0]);
 
-            // sdims fully define a problem. As sdims are parsed every time it's
-            // safe to process default tag for whatever number of inputs.
-            if (s.stag.empty()) s.stag = {{s.sdims.size(), "abx"}};
-
             check_correctness(s);
         }
     }
