@@ -32,16 +32,31 @@ void check_correctness(const settings_t &s) {
     for_(const auto &i_stag : s.stag)
     for_(const auto &i_dtag : s.dtag)
     for_(const auto &i_oflag : s.oflag)
+    for_(const auto &i_alg : s.alg)
+    for_(const auto &i_oscale : s.oscale)
+    for_(const auto &i_zero_points : s.zero_points)
+    for_(const auto &i_post_ops : s.post_ops)
     for (auto i_runtime_dim_mask : s.runtime_dim_mask) {
         reorder_conf_t reorder_conf {s.dims, i_stag, i_dtag};
         dt_conf_t iconf = dt2cfg(i_sdt);
         dt_conf_t oconf = dt2cfg(i_ddt);
+        attr_t attr(i_oscale, i_zero_points, i_post_ops);
+        handle_legacy_attr(attr, s.attr);
 
-        std::vector<float> attr_scale = {s.attr.oscale.scale};
-        auto &scale = s.attr.oscale.scale == 0 ? s.def_scale : attr_scale;
+        if (attr.oscale.policy == policy_t::PER_OC) {
+            fprintf(stderr,
+                    "ERROR: reorder driver: `per_oc` policy is not supported "
+                    "due to potential ambiguity. Please use one of `per_dim_0` "
+                    "or `per_dim_1` policies.\n"),
+                    fflush(stderr);
+            SAFE_V(FAIL);
+        }
+
+        std::vector<float> attr_scale = {attr.oscale.scale};
+        auto &scale = attr.oscale.scale == 0 ? s.def_scale : attr_scale;
 
         for (const auto &i_scale : scale) {
-            const prb_t p(reorder_conf, iconf, oconf, s.attr, s.alg, i_oflag,
+            const prb_t p(reorder_conf, iconf, oconf, attr, i_alg, i_oflag,
                     i_runtime_dim_mask, i_scale);
             std::stringstream ss;
             ss << p;
@@ -53,7 +68,7 @@ void check_correctness(const settings_t &s) {
             int status = doit(&p, &res);
 
             bool want_perf_report = false;
-            parse_result(res, want_perf_report, s.allow_unimpl, status, pstr);
+            parse_result(res, want_perf_report, status, pstr);
 
             if (want_perf_report && bench_mode & PERF) {
                 perf_report_t pr(s.perf_template);
@@ -69,20 +84,25 @@ int bench(int argc, char **argv) {
     driver_name = "reorder";
     using namespace parser;
     static settings_t s;
+    static const settings_t def {};
     for (; argc > 0; --argc, ++argv) {
         const bool parsed_options = parse_bench_settings(argv[0])
                 || parse_batch(bench, argv[0])
-                || parse_dt(s.sdt, argv[0], "sdt")
-                || parse_dt(s.ddt, argv[0], "ddt")
-                || parse_tag(s.stag, argv[0], "stag")
-                || parse_tag(s.dtag, argv[0], "dtag")
-                || parse_vector_option(s.oflag, str2flag, argv[0], "oflag")
+                || parse_dt(s.sdt, def.sdt, argv[0], "sdt")
+                || parse_dt(s.ddt, def.ddt, argv[0], "ddt")
+                || parse_tag(s.stag, def.stag, argv[0], "stag")
+                || parse_tag(s.dtag, def.dtag, argv[0], "dtag")
                 || parse_vector_option(
-                        s.runtime_dim_mask, atoi, argv[0], "runtime-dim-mask")
-                || parse_single_value_option(s.alg, str2alg, argv[0], "alg")
-                || parse_vector_option(s.def_scale, atof, argv[0], "def-scales")
+                        s.oflag, def.oflag, str2flag, argv[0], "oflag")
+                || parse_vector_option(s.runtime_dim_mask, def.runtime_dim_mask,
+                        atoi, argv[0], "runtime-dim-mask")
+                || parse_alg(s.alg, def.alg, str2alg, argv[0])
+                || parse_vector_option(
+                        s.def_scale, def.def_scale, atof, argv[0], "def-scales")
                 || parse_attr(s.attr, argv[0])
-                || parse_allow_unimpl(s.allow_unimpl, argv[0])
+                || parse_attr_oscale(s.oscale, argv[0])
+                || parse_attr_zero_points(s.zero_points, argv[0])
+                || parse_attr_post_ops(s.post_ops, argv[0])
                 || parse_perf_template(s.perf_template, s.perf_template_def,
                         s.perf_template_csv, argv[0])
                 || parse_reset(s, argv[0]);
