@@ -20,7 +20,7 @@
 
 #include "dnnl.h"
 
-#include "src/common/dnnl_thread.hpp"
+#include "tests/test_thread.hpp"
 
 #include "dnnl_common.hpp"
 #include "dnnl_memory.hpp"
@@ -92,7 +92,9 @@ static int compare(const prb_t *p, const dnn_mem_t &fp_mem,
     return r->state == FAILED ? FAIL : OK;
 }
 
-static int init_pd(const prb_t *p, dnnl_primitive_desc_t &spd, res_t *r) {
+static int init_pd(dnnl_engine_t engine, const prb_t *p,
+        dnnl_primitive_desc_t &spd, res_t *r, dir_t dir,
+        const_dnnl_primitive_desc_t hint) {
     dnnl_memory_desc_t data_d;
     dnnl_shuffle_desc_t sd;
 
@@ -106,57 +108,61 @@ static int init_pd(const prb_t *p, dnnl_primitive_desc_t &spd, res_t *r) {
                      &sd, prop_kind, &data_d, p->axis, p->group),
             WARN);
 
-    dnnl_primitive_desc_t hint = NULL;
+    dnnl_primitive_desc_t _hint = NULL;
+    auto cleanup_pd = [&]() { dnnl_primitive_desc_destroy(_hint); };
     if (p->dir & FLAG_BWD) {
-        dnnl_status_t init_fwd_status = dnnl_primitive_desc_create(
-                &hint, &sd, NULL, engine_tgt, NULL);
+        dnnl_status_t init_fwd_status
+                = dnnl_primitive_desc_create(&_hint, &sd, NULL, engine, NULL);
         if (init_fwd_status == dnnl_unimplemented)
             return r->state = UNIMPLEMENTED, OK;
         SAFE(init_fwd_status, WARN);
 
-        DNN_SAFE(dnnl_memory_desc_init_by_tag(&data_d, p->ndims, p->dims.data(),
-                         p->dt, dnnl_format_tag_any),
-                WARN);
+        DNN_SAFE_CLEAN(dnnl_memory_desc_init_by_tag(&data_d, p->ndims,
+                               p->dims.data(), p->dt, dnnl_format_tag_any),
+                WARN, cleanup_pd);
 
-        DNN_SAFE(dnnl_shuffle_backward_desc_init(
-                         &sd, &data_d, p->axis, p->group),
-                WARN);
+        DNN_SAFE_CLEAN(dnnl_shuffle_backward_desc_init(
+                               &sd, &data_d, p->axis, p->group),
+                WARN, cleanup_pd);
     }
 
     auto dnnl_attr = create_dnnl_attr(attr_t());
 
-    dnnl_status_t init_status = dnnl_primitive_desc_create(
-            &spd, &sd, dnnl_attr, engine_tgt, hint);
+    dnnl_status_t init_status
+            = dnnl_primitive_desc_create(&spd, &sd, dnnl_attr, engine, _hint);
 
-    dnnl_primitive_desc_destroy(hint);
+    dnnl_primitive_desc_destroy(_hint);
     dnnl_primitive_attr_destroy(dnnl_attr);
 
     if (init_status == dnnl_unimplemented) return r->state = UNIMPLEMENTED, OK;
     SAFE(init_status, WARN);
 
-    const char *impl_str = query_impl_info(spd);
-    BENCHDNN_PRINT(5, "oneDNN implementation: %s\n", impl_str);
+    r->impl_name = query_impl_info(spd);
+    BENCHDNN_PRINT(5, "oneDNN implementation: %s\n", r->impl_name.c_str());
 
     return OK;
 }
 
+void check_known_skipped_case(const prb_t *p, res_t *r) {
+    check_known_skipped_case_common({p->dt}, r);
+}
+
 int doit(const prb_t *p, res_t *r) {
     if (bench_mode == LIST) return r->state = LISTED, OK;
 
-    dnnl_primitive_desc_t spd;
-    SAFE(init_pd(p, spd, r), WARN);
-    if (r->state == SKIPPED || r->state == UNIMPLEMENTED) return OK;
+    check_known_skipped_case(p, r);
+    if (r->state == SKIPPED) return OK;
 
-    dnnl_primitive_t s;
-    DNN_SAFE(dnnl_primitive_create(&s, spd), WARN);
-    DNN_SAFE(dnnl_primitive_desc_destroy(spd), CRIT);
+    dnnl_primitive_t s {};
+    SAFE(init_prim(&s, init_pd, p, r), WARN);
+    if (r->state == SKIPPED || r->state == UNIMPLEMENTED) return OK;
 
     const_dnnl_primitive_desc_t const_pd;
     DNN_SAFE(dnnl_primitive_get_primitive_desc(s, &const_pd), CRIT);
 
     if (dnn_mem_t::check_mem_size(const_pd) != OK) {
         DNN_SAFE_V(dnnl_primitive_destroy(s));
-        return r->state = SKIPPED, OK;
+        return r->state = SKIPPED, r->reason = NOT_ENOUGH_RAM, OK;
     }
 
     const auto q = [&](int index = 0) -> const dnnl_memory_desc_t & {
@@ -171,13 +177,15 @@ int doit(const prb_t *p, res_t *r) {
     const auto fp = dnnl_f32;
     const auto tag = get_abx_tag(p->ndims);
 
-    dnn_mem_t src_fp(data_md, fp, tag, engine_tgt);
-    dnn_mem_t src_dt(data_md, engine_tgt);
+    const auto &test_engine = get_test_engine();
 
-    dnn_mem_t dst_fp(data_md, fp, tag, engine_tgt);
-    dnn_mem_t dst_dt(data_md, engine_tgt);
+    dnn_mem_t src_fp(data_md, fp, tag, test_engine);
+    dnn_mem_t src_dt(data_md, test_engine);
 
-    dnn_mem_t scratchpad_dt(scratchpad_md, engine_tgt);
+    dnn_mem_t dst_fp(data_md, fp, tag, test_engine);
+    dnn_mem_t dst_dt(data_md, test_engine);
+
+    dnn_mem_t scratchpad_dt(scratchpad_md, test_engine);
 
     SAFE(fill_src(p, src_dt, src_fp), WARN);
 
@@ -185,15 +193,16 @@ int doit(const prb_t *p, res_t *r) {
     const int o_arg = p->dir == FWD_D ? DNNL_ARG_DST : DNNL_ARG_DIFF_SRC;
 
     args_t args;
+
     args.set(i_arg, src_dt);
     args.set(o_arg, dst_dt);
     args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
-    DNN_SAFE(execute_and_wait(s, stream_tgt, args), WARN);
+    SAFE(execute_and_wait(s, args), WARN);
 
     if (bench_mode & CORR) {
         compute_shuffle(p, src_fp, dst_fp);
-        dnn_mem_t data(dst_dt, fp, tag, engine_tgt);
+        dnn_mem_t data(dst_dt, fp, tag, test_engine);
         SAFE(compare(p, dst_fp, data, r), WARN);
     }
 
