@@ -38,7 +38,7 @@ using params_w_engine_t = std::tuple<dnnl::engine::kind, params_t>;
 class memory_creation_test
     : public ::testing::TestWithParam<params_w_engine_t> {
 protected:
-    virtual void SetUp() override {
+    void SetUp() override {
         params_w_engine_t pwe
                 = ::testing::TestWithParam<decltype(pwe)>::GetParam();
 
@@ -117,6 +117,8 @@ auto cases_zero_dim = ::testing::Values(params_t {{2, 0, 1, 1}, fmt::nChw16c},
 
 auto cases_generic = ::testing::Values(params_t {{2, 15, 3, 2}, fmt::nChw16c},
         params_t {{2, 15, 3, 2, 4}, fmt::nCdhw8c},
+        params_t {{2, 9, 3}, fmt::OIw8o4i},
+        params_t {{2, 9, 3, 2}, fmt::OIhw8o4i},
         params_t {{2, 9, 3, 2}, fmt::OIhw8o8i},
         params_t {{2, 9, 3, 2}, fmt::OIhw8i16o2i},
         params_t {{2, 9, 3, 2}, fmt::OIhw8o16i2o},
@@ -124,15 +126,21 @@ auto cases_generic = ::testing::Values(params_t {{2, 15, 3, 2}, fmt::nChw16c},
         params_t {{2, 9, 3, 2}, fmt::OIhw16i16o},
         params_t {{2, 9, 3, 2}, fmt::OIhw4i16o4i},
         params_t {{2, 9, 3, 2}, fmt::OIhw2i8o4i},
+        params_t {{2, 9, 3, 2, 4}, fmt::OIdhw8o4i},
+        params_t {{2, 17, 9, 2}, fmt::gOIw8o4i},
         params_t {{3, 18, 9, 3, 2, 3}, fmt::gOIdhw4i16o4i},
         params_t {{2, 18, 8, 4, 2, 3}, fmt::gOIdhw2i8o4i},
         params_t {{1, 2, 9, 3, 3, 2}, fmt::gOIdhw4o4i},
+        params_t {{2, 9, 3, 2}, fmt::OIhw16i16o4i},
+        params_t {{2, 9, 3, 2}, fmt::OIhw16i16o2i},
         params_t {{2, 9, 4, 3, 2}, fmt::gOihw16o},
+        params_t {{2, 17, 9, 3, 2}, fmt::gOIhw8o4i},
         params_t {{1, 2, 9, 3, 2}, fmt::gOIhw8o8i},
         params_t {{1, 2, 9, 3, 2}, fmt::gOIhw4o4i},
         params_t {{1, 2, 9, 3, 2}, fmt::gOIhw8i8o},
         params_t {{2, 17, 9, 3, 2}, fmt::gOIhw4i16o4i},
         params_t {{2, 17, 9, 3, 2}, fmt::gOIhw2i8o4i},
+        params_t {{2, 17, 9, 3, 2, 4}, fmt::gOIdhw8o4i},
         params_t {{2, 16, 16, 3}, fmt::gOIw2i4o2i},
         params_t {{2, 8, 6, 3, 3}, fmt::gOIhw2i4o2i},
         params_t {{2, 14, 18, 3, 3, 4}, fmt::gOIdhw2i4o2i},
@@ -148,7 +156,9 @@ auto cases_generic = ::testing::Values(params_t {{2, 15, 3, 2}, fmt::nChw16c},
         params_t {{15, 16, 16, 3, 3}, fmt::Goihw8g},
         params_t {{2, 9, 3}, fmt::OIw2i8o4i},
         params_t {{2, 17, 9, 3}, fmt::gOIw2i8o4i},
-        params_t {{15, 16, 16, 3}, fmt::Goiw8g});
+        params_t {{15, 16, 16, 3}, fmt::Goiw8g},
+        params_t {{2, 17, 9, 3, 2}, fmt::gOIhw16i16o4i},
+        params_t {{2, 17, 9, 3, 2}, fmt::gOIhw16i16o2i});
 } // namespace
 
 INSTANTIATE_TEST_SUITE_P(TestMemoryCreationEF, memory_creation_test,
