@@ -19,7 +19,6 @@
 #include "dnnl_test_common.hpp"
 #include "gtest/gtest.h"
 
-#include "cpu_isa_traits.hpp"
 #include "dnnl.hpp"
 
 namespace dnnl {
@@ -212,9 +211,8 @@ protected:
         SKIP_IF(data_type == memory::data_type::bf16
                         && get_test_engine_kind() == engine::kind::gpu,
                 "GPU does not support bf16 data type.");
-        SKIP_IF(data_type == memory::data_type::bf16
-                        && !impl::cpu::mayiuse(impl::cpu::avx512_core),
-                "ISA does not support bf16 data type.");
+        SKIP_IF(unsupported_data_type(data_type),
+                "Engine does not support this data type.");
 
         p = ::testing::TestWithParam<decltype(p)>::GetParam();
 
