@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-## Copyright 2018-2020 Intel Corporation
+## Copyright 2018-2021 Intel Corporation
 ## SPDX-License-Identifier: Apache-2.0
 
 import os
@@ -15,9 +15,6 @@ from model import *
 from color import *
 from result import *
 
-def pad(x):
-  return round_up(x, ALIGNMENT)
-
 def main():
   # Parse the command line arguments
   cfg = parse_args(description='Performs inference on a dataset using the specified training result.')
@@ -35,25 +32,56 @@ def main():
   result_cfg = load_config(result_dir)
   cfg.features = result_cfg.features
   cfg.transfer = result_cfg.transfer
-  target_feature = 'hdr' if 'hdr' in cfg.features else 'ldr'
+  cfg.model    = result_cfg.model
+  main_feature = get_main_feature(cfg.features)
+  num_main_channels = len(get_dataset_channels(main_feature))
+
+  # Initialize the dataset
+  data_dir = get_data_dir(cfg, cfg.input_data)
+  image_sample_groups = get_image_sample_groups(data_dir, cfg.features)
+
+  # Initialize the model
+  model = get_model(cfg)
+  model.to(device)
+
+  # Load the checkpoint
+  checkpoint = load_checkpoint(result_dir, device, cfg.num_epochs, model)
+  epoch = checkpoint['epoch']
+  print('Epoch:', epoch)
+
+  # Initialize the transfer function
+  transfer = get_transfer_function(cfg)
+
+  # Iterate over the images
+  print()
+  output_dir = os.path.join(cfg.output_dir, cfg.input_data)
+  metric_sum = {metric : 0. for metric in cfg.metric}
+  metric_count = 0
+  model.eval()
 
   # Inference function
-  def infer(model, transfer, input, exposure):
+  def infer(input, exposure):
     x = input.clone()
 
     # Apply the transfer function
-    color = x[:, 0:3, ...]
-    if 'hdr' in cfg.features:
-      color *= exposure
-    color = transfer.forward(color)
-    x[:, 0:3, ...] = color
+    if transfer:
+      color = x[:, 0:num_main_channels, ...]
+      if main_feature == 'hdr':
+        color *= exposure
+      color = transfer.forward(color)
+      x[:, 0:num_main_channels, ...] = color
 
     # Pad the output
     shape = x.shape
-    x = F.pad(x, (0, pad(shape[3])-shape[3], 0, pad(shape[2])-shape[2]))
+    x = F.pad(x, (0, round_up(shape[3], model.alignment) - shape[3],
+                  0, round_up(shape[2], model.alignment) - shape[2]))
 
     # Run the inference
-    x = model(x)
+    if main_feature == 'sh1':
+      # Iterate over x, y, z
+      x = torch.cat([model(torch.cat((x[:, i:i+3, ...], x[:, 9:, ...]), 1)) for i in [0, 3, 6]], 1)
+    else:
+      x = model(x)
 
     # Unpad the output
     x = x[:, :, :shape[2], :shape[3]]
@@ -62,46 +90,35 @@ def main():
     x = torch.clamp(x, min=0.)
 
     # Apply the inverse transfer function
-    x = transfer.inverse(x)
-    if 'hdr' in cfg.features:
-      x /= exposure
-    else:
-      x = torch.clamp(x, max=1.)
+    if transfer:
+      x = transfer.inverse(x)
+      if main_feature == 'hdr':
+        x /= exposure
+      else:
+        x = torch.clamp(x, max=1.)
+        
     return x
 
   # Saves an image in different formats
-  def save_images(path, image, image_srgb):
+  def save_images(path, image, image_srgb, suffix=main_feature):
+    if suffix == 'sh1':
+      # Iterate over x, y, z
+      for i, axis in [(0, 'x'), (3, 'y'), (6, 'z')]:
+        save_images(path, image[:, i:i+3, ...], image_srgb[:, i:i+3, ...], 'sh1' + axis)
+      return
+
     image      = tensor_to_image(image)
     image_srgb = tensor_to_image(image_srgb)
-    filename_prefix = path + '.' + target_feature + '.'
+    filename_prefix = path + '.' + suffix + '.'
     for format in cfg.format:
       if format in {'exr', 'pfm', 'hdr'}:
+        # Transform to original range
+        if main_feature in {'sh1', 'nrm'}:
+          image = image * 2. - 1. # [0..1] -> [-1..1]
         save_image(filename_prefix + format, image)
       else:
         save_image(filename_prefix + format, image_srgb)
 
-  # Initialize the dataset
-  data_dir = get_data_dir(cfg, cfg.input_data)
-  image_sample_groups = get_image_sample_groups(data_dir, cfg.features)
-
-  # Initialize the model
-  model = UNet(get_num_channels(cfg.features))
-  model.to(device)
-
-  # Load the checkpoint
-  checkpoint = load_checkpoint(cfg, device, cfg.checkpoint, model)
-  epoch = checkpoint['epoch']
-
-  # Initialize the transfer function
-  transfer = get_transfer_function(cfg.transfer)
-
-  # Iterate over the images
-  print()
-  output_dir = os.path.join(cfg.output_dir, cfg.input_data)
-  metric_sum = {metric : 0. for metric in cfg.metric}
-  metric_count = 0
-  model.eval()
-
   with torch.no_grad():
     for group, input_names, target_name in image_sample_groups:
       # Create the output directory if it does not exist
@@ -118,27 +135,27 @@ def main():
 
       # Load the target image if it exists
       if target_name:
-        target = load_target_image(os.path.join(data_dir, target_name), cfg.features)
+        target = load_image_features(os.path.join(data_dir, target_name), main_feature)
         target = image_to_tensor(target, batch=True).to(device)
-        target_srgb = transform_feature(target, target_feature, 'srgb', tonemap_exposure)
+        target_srgb = transform_feature(target, main_feature, 'srgb', tonemap_exposure)
 
       # Iterate over the input images
       for input_name in input_names:
         print(input_name, '...', end='', flush=True)
 
         # Load the input image
-        input = load_input_image(os.path.join(data_dir, input_name), cfg.features)
+        input = load_image_features(os.path.join(data_dir, input_name), cfg.features)
 
         # Compute the autoexposure value
-        exposure = autoexposure(input) if 'hdr' in cfg.features else 1.
+        exposure = autoexposure(input) if main_feature == 'hdr' else 1.
 
         # Infer
         input = image_to_tensor(input, batch=True).to(device)
-        output = infer(model, transfer, input, exposure)
+        output = infer(input, exposure)
 
-        input = input[:, 0:3, ...] # keep only the color
-        input_srgb  = transform_feature(input,  target_feature, 'srgb', tonemap_exposure)
-        output_srgb = transform_feature(output, target_feature, 'srgb', tonemap_exposure)
+        input = input[:, 0:num_main_channels, ...] # keep only the main feature
+        input_srgb  = transform_feature(input,  main_feature, 'srgb', tonemap_exposure)
+        output_srgb = transform_feature(output, main_feature, 'srgb', tonemap_exposure)
 
         # Compute metrics
         metric_str = ''
@@ -153,7 +170,7 @@ def main():
 
         # Save the input and output images
         output_name = input_name + '.' + cfg.result
-        if cfg.checkpoint:
+        if cfg.num_epochs:
           output_name += f'_{epoch}'
         if cfg.save_all:
           save_images(os.path.join(output_dir, input_name), input, input_srgb)
