@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-## Copyright 2018-2020 Intel Corporation
+## Copyright 2018-2021 Intel Corporation
 ## SPDX-License-Identifier: Apache-2.0
 
 import os
@@ -18,23 +18,29 @@ import tza
 def main():
   # Parse the command line arguments
   cfg = parse_args(description='Preprocesses training and validation datasets.')
+  main_feature = get_main_feature(cfg.features)
+  num_main_channels = len(get_dataset_channels(main_feature))
 
   # Initialize the PyTorch device
   device = init_device(cfg)
 
   # Initialize the transfer function
-  transfer = get_transfer_function(cfg.transfer)
+  transfer = get_transfer_function(cfg)
 
   # Returns a preprocessed image (also changes the original image!)
   def preprocess_image(image, exposure):
-    color = image[..., 0:3]
-    color = torch.from_numpy(color).to(device)
-    if 'hdr' in cfg.features:
-      color *= exposure
-    color = transfer.forward(color)
-    color = torch.clamp(color, max=1.)
-    color = color.cpu().numpy()
-    image[..., 0:3] = color
+    # Apply the transfer function
+    if transfer:
+      color = image[..., 0:num_main_channels]
+      color = torch.from_numpy(color).to(device)
+      if main_feature == 'hdr':
+        color *= exposure
+      color = transfer.forward(color)
+      color = torch.clamp(color, max=1.)
+      color = color.cpu().numpy()
+      image[..., 0:num_main_channels] = color
+
+    # Convert to FP16
     return np.nan_to_num(image.astype(np.float16))
 
   # Preprocesses a group of input and target images at different SPPs
@@ -43,10 +49,10 @@ def main():
 
     # Load the target image
     print(target_name)
-    target_image = load_target_image(os.path.join(input_dir, target_name), cfg.features)
+    target_image = load_image_features(os.path.join(input_dir, target_name), main_feature)
 
     # Compute the autoexposure value
-    exposure = autoexposure(target_image) if 'hdr' in cfg.features else 1.
+    exposure = autoexposure(target_image) if main_feature == 'hdr' else 1.
 
     # Preprocess the target image
     target_image = preprocess_image(target_image, exposure)
@@ -58,7 +64,7 @@ def main():
     for input_name in input_names:
       # Load the image
       print(input_name)
-      input_image = load_input_image(os.path.join(input_dir, input_name), cfg.features)
+      input_image = load_image_features(os.path.join(input_dir, input_name), cfg.features)
 
       if input_image.shape[0:2] != target_image.shape[0:2]:
         error('the input and target images have different sizes')
@@ -77,26 +83,16 @@ def main():
   # Preprocesses a dataset
   def preprocess_dataset(data_name):
     input_dir = get_data_dir(cfg, data_name)
-    output_dir = get_preproc_data_dir(cfg, data_name)
-    print('Dataset:', input_dir)
-
-    # Check whether the dataset exists
+    print('\nDataset:', input_dir)
     if not os.path.isdir(input_dir):
-      print('Does not exist')
+      print('Not found')
       return
 
-    # Create the output directory if it doesn't exist
-    if os.path.isdir(output_dir):
-      if cfg.clean:
-        shutil.rmtree(output_dir)
-      else:
-        print('Skipping, already preprocessed')
-        return
+    # Create the output directory
+    output_name = data_name + '.' + WORKER_UID
+    output_dir = os.path.join(cfg.preproc_dir, output_name)
     os.makedirs(output_dir)
 
-    # Save the config
-    save_config(output_dir, cfg)
-
     # Preprocess image sample groups
     sample_groups = get_image_sample_groups(input_dir, cfg.features)
     tza_filename = os.path.join(output_dir, 'images.tza')
@@ -110,11 +106,14 @@ def main():
     samples_filename = os.path.join(output_dir, 'samples.json')
     save_json(samples_filename, samples)
 
+    # Save the config
+    save_config(output_dir, cfg)
+
   # Preprocess all datasets
   with torch.no_grad():
     for dataset in [cfg.train_data, cfg.valid_data]:
-      print()
-      preprocess_dataset(dataset)
+      if dataset:
+        preprocess_dataset(dataset)
 
 if __name__ == '__main__':
   main()
\ No newline at end of file
